/*
 * CifCrystal.h
 *
 * Copyright Notice: see copyright.txt
 *
 *  Date: 3/8/2012 (D/M/Y)
 *  Author: Dmitrij Lioubartsev
 *  Email: dmitrijl42@gmail.com
 *
 *  Main class to hold crystal structure information.
 *  Also contains the methods for manipulating the data.
 *  It stores and processes symmetries, duplicates atoms
 *  into a supercell, transforms atom coordinates into cartesian
 *  (from fractional), and calculates atomic bonds.
 */

#ifndef CIFCRYSTAL_H_
#define CIFCRYSTAL_H_

#include <string>
#include <vector>
#include <set>
using namespace std;

#include "Atom.h"



class CifCrystal {
public:
	/*
	 * Represents an atomic bond between two atoms. Ints are indices in
	 * main Atom storage vector atoms.
	 */
	struct Bond {
		unsigned int a1;
		unsigned int a2;
	};

private:
	double a,b,c; // the size of the structure, the side lengths
	double alpha,beta,gamma; // The angles of the structure
	bool isSet[6];
	vector<string> symmetries; //a string array of the symmetries
	//set<Atom, AtomCmpRel> atoms;
	vector<Atom> atoms; //an array of all the atoms
	unsigned int numEl, numx, numy, numz;
	//used for properly keeping track of number of atoms with what
	//values assigned

	vector<Bond> bonds; // a vector of bonds. Not used in any calculations, but
						//it is an easy way of creating the .smol file later.

	double aNew, bNew, cNew; //these are set after the supercell duplication is ran.
	int aDup, bDup, cDup;


public:
	//constructor and destructor
	CifCrystal();
	virtual ~CifCrystal(); //does nothing
	//methods

	/*
	 * These simply set the value. If it has already been
	 * set, they return false and don't change the value.
	 */
	bool seta(double value);
	bool setb(double value);
	bool setc(double value);
	bool setalpha(double value);
	bool setbeta(double value);
	bool setgamma(double value);

	bool* areSet();

	//getters for the above
	//the _ are needed because getc apparently
	//is a macro in std or something -_-
	double get_a();
	double get_b();
	double get_c();
	double get_alpha();
	double get_beta();
	double get_gamma();
	double getNewa();
	double getNewb();
	double getNewc();
	int getDupa();
	int getDupb();
	int getDupc();

	/*
	 * Symmetry methods here
	 */
	// This adds a symmetry string to the list, without any checks.
	void addSymmetry(string value);

	/*
	 * Atom methods here
	 */
	//set methods
	void setAx(double x);
	void setAy(double y);
	void setAz(double z);
	void setAElement(string name);
	//number of atoms currently in the list
	unsigned int numAtoms();
	//returns atom with given index
	//Atom* getAtom(unsigned int index);
	vector<Atom>* getAtomList();
	vector<Bond>* getBondList();

	bool okData();

	/*
	 * Explanation of how storage of atoms is handled:
	 * "atoms" is the main vector that holds the atoms.
	 * However, because of how atom data is written in
	 * the .cif file, its not possible to just have an
	 * "addAtom()" method like with symmetry. So methods
	 * of setting atom properties are needed, and thats what
	 * setAx() etc. is. When setAx() is called, it checks
	 * numx, which is how many times setAx() has been called.
	 * If there still is an atom in "atoms" which does not
	 * have its "x" coordinate set, that atom will get it set,
	 * otherwise it creates a new Atom and sets its 'x' property.
	 */

	/*
	 * A method for testing purposes
	 */
	void printThis();

	/*
	 * This method processes the symmetries and removes them,
	 * while adding Atoms to the list. Multiple enteties of the same
	 * Atom are not added, uses Atom operator == to compare.
	 */
	void processSymmetries();

	/*
	 * Takes the current atoms and duplicates them in the x,y,z directions,
	 * depending on the r_min value. Also sets the aNew, bNew and cNew variables.
	 */
	void makeSuperCell(double r_min);

	/*
	 * Takes the current atoms, that should be in the relative coordinates,
	 * and transforms them to Euclidian coordinates. Also sets a,b and c
	 * to the new superCell lengths, but in euclidian lengths.
	 */
	void makeTransformations();


	void makeBonds();

};

#endif /* CIFCRYSTAL_H_ */
